local function update(self, dt)
  self.x = self.x + self.vx * dt
  self.y = self.y + self.vy * dt
  
  for i=1,#self.points do
    local point = self.points[i]
    
    local dphi = math.random(-180,180) * dt
    local dtheta = math.random(-180,180) * dt
    
    point.phi = point.phi + dphi
    point.theta = point.theta + dtheta
  end
end

local function draw(self, colors, debug)
  if debug then
    love.graphics.setColor(colors[1])
    love.graphics.circle("line",self.x,self.y,self.r)
  else 
    for i=1,#self.points do
      local point = self.points[i]
      
      local phi = math.rad(point.phi)
      local theta = math.rad(point.theta)
      local dx = math.sin(theta)*math.cos(phi)*self.r
      local dy = math.cos(theta)*self.r
      
      love.graphics.setColor(colors[point.color])
      love.graphics.points(self.x+dx,self.y+dy)
    end
  end
end

local stars = {}

function stars:new(x,y,r,vx,vy)
  local star = {}
  
  star.update = update
  star.draw = draw
  
  star.kill = false
  
  star.x = x or 0
  star.y = y or 0
  star.vx = vx or 0
  star.vy = vy or 0
  
  star.r = r or 100
  star.m = star.r / 2
  
  local nbPoints = math.floor(math.pow(star.r*1000,0.7))
  star.points = {}
  for i=1, nbPoints do
    local point = {}
    point.phi = math.random(1,360) -- horizontal
    point.theta = math.random(1,180) -- vertical
    point.color = math.random(1,5)
    table.insert(star.points,point)
  end
  table.insert(self.all,star)
  return star
end

function stars:init()
  self.all = {}
  self.colors = {
    {255,0,0},
    {255,206,0},
    {255,141,0},
    {253,255,7},
    {255,255,255},
  }
end

function stars:explode(particles,star,power)  
  local ratio = 1/3
  for i=1,#star.points do
    if math.random() > ratio then
      local point = star.points[i]
      -- calculate
      local phi = math.rad(point.phi)
      local theta = math.rad(point.theta)
      local dx = math.sin(theta)*math.cos(phi)*star.r
      local dy = math.cos(theta)*star.r
      local x = star.x + dx
      local y = star.y + dy
      local v = math.random(1,power)
      local a = math.rad(math.random(1,360))
      local vx = star.vx + v * math.cos(a)
      local vy = star.vy + v * math.sin(a)
      local c = self.colors[point.color]
      particles:new(x, y, vx, vy, c)
    end
  end
end

function stars:collision(particles,a,b)
  local pow = 10
  local dr = math.pow(math.pow(a.r,pow) + math.pow(b.r,3),1/(1.5*pow))
  if a.r > b.r then
    a.r = a.r + dr
    self:explode(particles,b,100)
    b.kill = true
  else
    b.r = b.r + dr
    self:explode(particles,a,100)
    a.kill = true
  end
end

function stars:update(particles,physics,blackholes,dt)
  local marked = {}
  
  for i=#self.all,1,-1 do
    local star = self.all[i]
    marked[i] = true
    star:update(dt) -- velocity only
    
    for i=1,#blackholes.all do
      local blackhole = blackholes.all[i]
      local collision = physics:gravity(star, blackhole, dt)
      if collision then
        self:explode(particles,star,100)
        local dx = blackhole.x - star.x
        local dy = blackhole.y - star.y
        local d = math.sqrt(dx*dx + dy*dy)
        local newR = d - blackhole.r - 2
        star.r = newR
        local nbPoints = math.floor(math.pow(star.r*1000,0.7))
        for j=1,#star.points-nbPoints do
          table.remove(star.points,j)
        end
        if newR < 10 then
          star.kill = true
        end
      end
    end    
    
    
    for j=1,#self.all do
      if marked[j] ~= true then
        local other = self.all[j]
        local collision = physics:gravity(star,other,dt,true)
        if collision then 
          self:collision(particles,star,other) 
        end
      end
    end
    
    if star.kill then
      table.remove(self.all, i)
    end
  end
end

function stars:draw(camera,w,h,debug)
  for i=1,#self.all do
    local star = self.all[i]
    if star.x + star.r > camera.x 
    and star.x - star.r < camera.x + w / camera.scale
    and star.y + star.r > camera.y 
    and star.y - star.r < camera.y + h / camera.scale
    then 
      star:draw(self.colors,debug)
    end
  end
end

return stars